/* vim:et:ts=2
Fivegeeks kitchen clock for PalmOS
*/

#include <Pilot.h>
#include <System/SerialMgr.h>
#include "kitchenclock.h"
#include <stdarg.h>

#define E(X) ErrFatalDisplayIf(X, "error")

UInt serialmgr;
UInt autooff;
FormPtr form;
WinHandle win, dwin;
int formup = 0;

void swap() {
  RectangleType r;
  r.topLeft.x = 0;
  r.topLeft.y = 0;
  r.extent.x = 160;
  r.extent.y = 160;
  WinCopyRectangle(win, dwin, &r, 0, 0, scrCopy);
  WinEraseWindow();
}

static Boolean FormHandleEvent(EventPtr e) {
  switch (e->eType) {
  case frmOpenEvent:
    WinSetDrawWindow(win);
    swap();
    swap();
    formup = 1;
    return true;
  }

  return false;
}

int doevents() {
  while (EvtEventAvail()) {
    EventType ev;
    EvtGetEvent(&ev, 0);
    if (SysHandleEvent(&ev)) continue;
    if (ev.eType == frmLoadEvent) {
      form = FrmInitForm(ev.data.frmLoad.formID);
      FrmSetActiveForm(form);
      switch (ev.data.frmLoad.formID) {
      case ID_form:
	        FrmSetEventHandler(form, FormHandleEvent);
	        break;
	      }
      continue;
    }
    if (ev.eType == appStopEvent) return -1;
    FrmDispatchEvent(&ev);
  }
  return 0;
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags) {
  EventType e;
  Word error;

  if (cmd == sysAppLaunchCmdNormalLaunch) {
    FrmGotoForm(ID_form);
    E(SysLibFind("Serial Library", &serialmgr));
    E(SerOpen(serialmgr, 0, 19200));
    autooff = SysSetAutoOffTime(0);
    win = WinCreateOffscreenWindow(160, 160, screenFormat, &error);
    WinEnableWindow(win);
    dwin = WinGetDisplayWindow();
  
    while (1) {
      Err e;
      char c;
      int n, x, y;
#define SIZE 80
      char s[SIZE], buf[80], *p, *sp1 = NULL, *sp2 = NULL, *sp3 = NULL, *sp4 = NULL;
      RectangleType r;

      while (1) {
        if (doevents() == -1) goto quit;
        if (!formup) continue;
        if (SerReceive(serialmgr, &c, 1, 10, &e) == 1) break;
        if (e == serErrLineErr) SerClearErr(serialmgr);
      }

      n = x = c;
      p = s;
      E(n > (SIZE - 1));
      while (x > 0) {
        ULong c = SerReceive(serialmgr, p, x, -1, &e);
        if (e == serErrLineErr) SerClearErr(serialmgr);
        x -= c;
        p += c;
      }
      s[n] = '\0';

      sp1 = StrChr(s, ' ');
      if (sp1) {
        sp2 = StrChr(sp1 + 1, ' ');
        if (sp2) {
          sp3 = StrChr(sp2 + 1, ' ');
          if (sp3) {
            sp4 = StrChr(sp3 + 1, ' ');
          }
        }
      }

      switch (s[0]) {
      case 'd': /* "d" - draw to display and clear */
        swap();
        break;
      case 's': /* "sx y String" - draw string */
        E(sp2 == NULL);
        WinDrawChars(sp2+1, StrLen(sp2+1), StrAToI(s+1), StrAToI(sp1+1));
        break;
      case 'l': /* "lx y x y" - draw line */
        E(sp3 == NULL);
        WinDrawLine(StrAToI(s+1), StrAToI(sp1+1), StrAToI(sp2+1), StrAToI(sp3+1));
        break;
      case 'r': /* "rx y w h corner" - draw rectangle */
        E(sp4 == NULL);
        r.topLeft.x = StrAToI(s+1);
        r.topLeft.y = StrAToI(sp1+1);
        r.extent.x = StrAToI(sp2+1);
        r.extent.y = StrAToI(sp3+1);
        WinDrawRectangle(&r, StrAToI(sp4+1));
        break;
      case 'f': /* "fx y w h corner" - fill rectangle */
        E(sp4 == NULL);
        r.topLeft.x = StrAToI(s+1);
        r.topLeft.y = StrAToI(sp1+1);
        r.extent.x = StrAToI(sp2+1);
        r.extent.y = StrAToI(sp3+1);
        WinFillRectangle(&r, StrAToI(sp4+1));
        break;
      }
    }
    quit:

    WinDeleteWindow(win, 1);
    E(SerClose(serialmgr));
    SysSetAutoOffTime(autooff);
  } else {
    return sysErrParamErr;
  }
  return 0;
}




